package ink.wgink.module.form.service.design.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.ParamsException;
import ink.wgink.exceptions.PropertiesException;
import ink.wgink.exceptions.base.SystemException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.form.consts.IFormDesignConst;
import ink.wgink.module.form.enums.design.FormFieldTypeEnum;
import ink.wgink.module.form.pojo.vos.design.FormDesignVO;
import ink.wgink.module.form.pojo.vos.design.FormFieldVO;
import ink.wgink.module.form.pojo.vos.design.FormVO;
import ink.wgink.module.form.service.design.IFormDesignService;
import ink.wgink.module.form.service.design.IFormFieldService;
import ink.wgink.module.form.service.design.IFormService;
import ink.wgink.properties.form.FormProperties;
import ink.wgink.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description: 表单设计
 * @Author: WenG
 * @Date: 2022/3/7 10:53
 * @Version: 1.0
 **/
@Service
public class FormDesignServiceImpl extends DefaultBaseService implements IFormDesignService {

    @Autowired
    private FormProperties formProperties;
    @Autowired
    private IFormService formService;
    @Autowired
    private IFormFieldService formFieldService;

    @Override
    public void save(FormDesignVO formDesignVO) {
        if (StringUtils.isBlank(formProperties.getTemplatePath())) {
            throw new PropertiesException("未配置模板路径");
        }
        List<FormFieldVO> formFields = new ArrayList<>();
        setFormFieldList(formFields, formDesignVO.getData());
        if (formFields.isEmpty()) {
            throw new ParamsException("表单字段不能为空");
        }
        // 时间戳表单编码
        String formCode = DateUtil.getSdfTimes();
        // df：dynamicForm 动态表单
        String tableName = "df_" + formCode;

        LOG.debug("保存表单");
        FormVO formVO = new FormVO();
        formVO.setFormName(formDesignVO.getFormName());
        formVO.setFormCode(formCode);
        formVO.setFormSummary(formDesignVO.getFormSummary());
        formVO.setFormTableName(tableName);
        formVO.setFormVersion(1);
        formVO.setFormSourceData(JSON.toJSONString(formDesignVO.getData()));
        formVO.setFormPageCode(formDesignVO.getCode());
        String formId = formService.saveReturnId(formVO);

        LOG.debug("保存表单字段");
        formFields.forEach(formFieldVO -> {
            formFieldVO.setFormId(formId);
            formFieldService.save(formFieldVO);
        });

        LOG.debug("建表");
        String createTableSQL = getCreateTableSQL(tableName, formFields);
        formService.createDynamicForm(createTableSQL);

        LOG.debug("创建静态页面");
        File formFolder = new File(formProperties.getTemplatePath());
        if (!formFolder.exists()) {
            formFolder.mkdirs();
        }
        try (
                FileOutputStream fileOutputStream = new FileOutputStream(formProperties.getTemplatePath() + formCode + ".ftl");
                OutputStreamWriter outputStreamWriter = new OutputStreamWriter(fileOutputStream, ISystemConstant.CHARSET_UTF8);
                BufferedWriter bufferedWriter = new BufferedWriter(outputStreamWriter)
        ) {
            bufferedWriter.write(formDesignVO.getCode());
            bufferedWriter.flush();
        } catch (IOException e) {
            LOG.error(e.getMessage(), e);
            throw new SystemException("创建模板文件异常");
        }


    }

    /**
     * 建表SQL
     *
     * @param tableName
     * @param formFieldVOs
     * @return
     */
    private String getCreateTableSQL(String tableName, List<FormFieldVO> formFieldVOs) {
        StringBuffer createTableSB = new StringBuffer("CREATE TABLE `").append(tableName).append("`(");
        createTableSB.append("`id` bigint(20) unsigned NOT NULL AUTO_INCREMENT,");
        createTableSB.append("`uid` char(36) NOT NULL,");
        for (FormFieldVO formFieldVO : formFieldVOs) {
            createTableSB.append("`").append(formFieldVO.getFieldName()).append("` ").append(formFieldVO.getFieldType()).append(" NOT NULL COMMENT '").append(formFieldVO.getFieldExplain()).append("',");
        }
        createTableSB.append("`creator` char(36) DEFAULT NULL,");
        createTableSB.append("`gmt_create` datetime DEFAULT NULL,");
        createTableSB.append("`modifier` char(36) DEFAULT NULL,");
        createTableSB.append("`gmt_modified` datetime DEFAULT NULL,");
        createTableSB.append("`is_delete` int(2) DEFAULT '0',");
        createTableSB.append("PRIMARY KEY (`id`)");
        createTableSB.append(") ENGINE=InnoDB DEFAULT CHARSET=utf8;");
        return createTableSB.toString();
    }

    /**
     * 设置表单字段列表
     *
     * @param formFields
     * @param dataJsonArray
     */
    private void setFormFieldList(List<FormFieldVO> formFields, JSONArray dataJsonArray) {
        formFields = formFields == null ? new ArrayList<>() : formFields;
        for (int i = 0; i < dataJsonArray.size(); i++) {
            JSONObject dataJsonObject = dataJsonArray.getJSONObject(i);
            String id = dataJsonObject.getString(IFormDesignConst.ID);
            String index = dataJsonObject.getString(IFormDesignConst.INDEX);
            String label = dataJsonObject.getString(IFormDesignConst.LABEL);
            String tag = dataJsonObject.getString(IFormDesignConst.TAG);
            // grid特殊，需要递归处理
            if (StringUtils.equals(FormFieldTypeEnum.GRID.getType(), tag)) {
                JSONArray columnJsonArray = dataJsonObject.getJSONArray(IFormDesignConst.COLUMNS);
                for (int j = 0; j < columnJsonArray.size(); j++) {
                    JSONObject columnJsonObject = columnJsonArray.getJSONObject(j);
                    JSONArray listJsonArray = columnJsonObject.getJSONArray(IFormDesignConst.LIST);
                    if (!listJsonArray.isEmpty()) {
                        setFormFieldList(formFields, listJsonArray);
                    }
                }
            } else {
                FormFieldVO formField = new FormFieldVO();
                formField.setFieldName(id);
                formField.setFieldTag(tag);
                formField.setFieldExplain(label);
                formField.setFieldType(getSingleFieldType(tag));
                formFields.add(formField);
            }
        }
    }

    private String getSingleFieldType(String fieldTag) {
        if (StringUtils.equals(FormFieldTypeEnum.INPUT.getType(), fieldTag)) {
            return "VARCHAR(255)";
        } else if (StringUtils.equals(FormFieldTypeEnum.NUMBER_INPUT.getType(), fieldTag)) {
            return "DOUBLE(11,4)";
        } else if (StringUtils.equals(FormFieldTypeEnum.PASSWORD.getType(), fieldTag)) {
            return "VARCHAR(255)";
        } else if (StringUtils.equals(FormFieldTypeEnum.RADIO.getType(), fieldTag)) {
            return "VARCHAR(255)";
        } else if (StringUtils.equals(FormFieldTypeEnum.RADIO_DICTIONARY.getType(), fieldTag)) {
            return "VARCHAR(255)";
        } else if (StringUtils.equals(FormFieldTypeEnum.SELECT.getType(), fieldTag)) {
            return "VARCHAR(255)";
        } else if (StringUtils.equals(FormFieldTypeEnum.SELECT_DICTIONARY.getType(), fieldTag)) {
            return "VARCHAR(255)";
        } else if (StringUtils.equals(FormFieldTypeEnum.CHECKBOX.getType(), fieldTag)) {
            return "VARCHAR(255)";
        } else if (StringUtils.equals(FormFieldTypeEnum.CHECKBOX_DICTIONARY.getType(), fieldTag)) {
            return "VARCHAR(255)";
        } else if (StringUtils.equals(FormFieldTypeEnum.TEXTAREA.getType(), fieldTag)) {
            return "TEXT";
        } else if (StringUtils.equals(FormFieldTypeEnum.CAROUSEL.getType(), fieldTag)) {
            return "VARCHAR(300)";
        } else if (StringUtils.equals(FormFieldTypeEnum.FILE.getType(), fieldTag)) {
            return "VARCHAR(300)";
        } else if (StringUtils.equals(FormFieldTypeEnum.IMAGE.getType(), fieldTag)) {
            return "VARCHAR(300)";
        } else if (StringUtils.equals(FormFieldTypeEnum.RATE.getType(), fieldTag)) {
            return "DOUBLE(11,2)";
        } else if (StringUtils.equals(FormFieldTypeEnum.DATE.getType(), fieldTag)) {
            return "VARCHAR(255)";
        } else if (StringUtils.equals(FormFieldTypeEnum.SLIDER.getType(), fieldTag)) {
            return "DOUBLE(11,2)";
        } else if (StringUtils.equals(FormFieldTypeEnum.SWITCH.getType(), fieldTag)) {
            return "INT(1)";
        } else if (StringUtils.equals(FormFieldTypeEnum.LABEL_GENERATION.getType(), fieldTag)) {
            return "VARCHAR(255)";
        } else if (StringUtils.equals(FormFieldTypeEnum.EDITOR.getType(), fieldTag)) {
            return "LONGTEXT";
        } else if (StringUtils.equals(FormFieldTypeEnum.SIGN.getType(), fieldTag)) {
            return "VARCHAR(255)";
        }
        return null;
    }


}
