package ink.wgink.module.form.service.report.impl;

import freemarker.cache.FileTemplateLoader;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.base.SystemException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.form.service.report.IFormReportService;
import ink.wgink.properties.form.FormProperties;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Map;

/**
 * @ClassName: FormReportServiceImpl
 * @Description: 表单上报
 * @Author: wanggeng
 * @Date: 2022/3/10 7:33 PM
 * @Version: 1.0
 */
@Service
public class FormReportServiceImpl extends DefaultBaseService implements IFormReportService {

    @Autowired
    private FormProperties formProperties;
    @Autowired
    private FreeMarkerConfigurer freeMarkerConfigurer;

    @Override
    public void get(String formCode, HttpSession httpSession, HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) {
        httpServletResponse.setContentType("text/html; charset=" + ISystemConstant.CHARSET_UTF8);
        try {
            Template template = getTemplate(formCode + ".ftl");
            PrintWriter out = httpServletResponse.getWriter();
            Map<String, Object> modal = getHashMap(2);
            template.process(modal, out);
        } catch (IOException e) {
            httpServletResponse.setStatus(HttpStatus.NOT_FOUND.value());
        } catch (TemplateException e) {
            LOG.error(e.getMessage(), e);
            throw new SystemException("模板错误");
        }
    }

    private Configuration getConfiguration() throws IOException {
        freeMarkerConfigurer.setDefaultEncoding(ISystemConstant.CHARSET_UTF8);
        FileTemplateLoader fileTemplateLoader = new FileTemplateLoader(new File(formProperties.getTemplatePath()));
        Configuration configuration = freeMarkerConfigurer.getConfiguration();
        configuration.setTemplateLoader(fileTemplateLoader);
        return configuration;
    }

    private Template getTemplate(String fileName) throws IOException {
        return getConfiguration().getTemplate(fileName, ISystemConstant.CHARSET_UTF8);
    }

}
